//+------------------------------------------------------------------+

#property   description "ZigZagPoints.mq4"
#property   description "ZigZag Point Data Collection Script"
#property   version     "1.001"
#property   strict
#property   script_show_inputs

// Date Range Parameters
input
   datetime
      dtStartDateTime      = D'2013.01.01 00:00'   // Begining of Date/Time Period under review
   ,  dtStopDateTime       = D'2013.12.31 23:59'   // End Date/Time of Period under review
	;
	
// ZigZag Parameters
input
   int
      intZigZagDepth       = 12                    // ZigZag Depth
   ,  intZigZagDeviation   = 5                     // ZigZag Deviation
   ,  intBackstep          = 3                     // ZigZag Backstep
   ;
   
// CSV File Parameters
input
   string
      strCSVFileName       = "ZigZagPoints.csv"    // CSV Filename in "MQL4\Files" Folder
   ,  strCSVHeader         = "Count,Type,Price,DateTime" // CSV Header to be used
   ,  strCSVDelimiter      = ","                   // Delimiter to be used in CSV File
   ,  strZZHigh            = "H"                   // String to Identify Highs in CSV File
   ,  strZZLow             = "L"                   // String to Identify Lows in CSV File
   ,  strZZUnknown         = "?"                   // String to Identify Unknowns in CSV File
   ;
input
   int
      intDigits            = 5                     // Number of Digits of Doubles in CSV File
   ;

// Script Program Start Function                                    |
void OnStart()
{
	// Find Bar Shift of Start and End of the Date/Time Range
	int
	   intStartBarShift  = iBarShift( NULL, 0, dtStartDateTime, false )
	,  intStopBarShift   = iBarShift( NULL, 0, dtStopDateTime, false )
	;
	
	// Make sure the Shifts are valid
	if(   ( intStartBarShift   >= 0                 )  &&
	      ( intStopBarShift    >= 0                 )  &&
	      ( intStartBarShift   >= intStopBarShift   )     )
	{
	   // Open CSV File
	   int
	      intCSVFileHandle
	         = FileOpen(
	            strCSVFileName,
	            FILE_CSV | FILE_WRITE | FILE_ANSI,
	            strCSVDelimiter );
	   
	   if( intCSVFileHandle != INVALID_HANDLE )
	   {
	      // WRite CSV File Header
	      FileWrite( intCSVFileHandle, strCSVHeader );
	      
	      // Initialize ZigZag Counter
	      int intZZCount = 0;
	      
	      // Loop through the Bar range under review
	      for( int
	         intBarShift = intStartBarShift;
	         intBarShift >= intStopBarShift;
	         intBarShift-- )
	      {
	         // Collect the ZigZag Data
	         double
	            dblZigZagPoint
	               = iCustom( NULL, 0, "ZigZag",
	                  intZigZagDepth, intZigZagDeviation, intBackstep,
				         0, intBarShift )
            ,  dblZigZagHigh
	               = iCustom( NULL, 0, "ZigZag",
	                  intZigZagDepth, intZigZagDeviation, intBackstep,
				         1, intBarShift )
            ,  dblZigZagLow
	               = iCustom( NULL, 0, "ZigZag",
	                  intZigZagDepth, intZigZagDeviation, intBackstep,
				         2, intBarShift )
            ;
            
            // Test if a ZigZag Point is found
            if( dblZigZagPoint > 0 )
            {
               intZZCount++;  // Increment ZigZag Counter
               
               // Test if it is a High, Low or Unknown
               string strZZType = strZZUnknown;
               if( dblZigZagHigh > 0 )    strZZType = strZZHigh;
               else
                  if( dblZigZagLow > 0 )  strZZType = strZZLow;
               
               // Output the ZigZag Point Data to CSV File
               FileWrite( intCSVFileHandle,
                  intZZCount, strZZType,
                  DoubleToString( dblZigZagPoint, intDigits ),
                  Time[ intBarShift ] );
            }
	      }
	      
	      // Close CSV File
	      FileClose( intCSVFileHandle );
	   }
	   else
	      Print( "Unable to Create CSV File!" );
	}
	else
	   Print( "Invalid Start/Stop Date/Time Range!" );
}

//+------------------------------------------------------------------+